@@ -1,5 +1,5 @@
 /* GKrellMSS - GKrellM Sound Scope
-|  Copyright (C) 2002 Bill Wilson
+|  Copyright (C) 2002-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |
@@ -47,7 +47,8 @@ static GdkBitmap	*button_sweep_up_mask,
 static GkrellmDecal	*decal_sweep_up,
 					*decal_sweep_dn;
 
-static GkrellmDecal	*decal_mode_button;
+static GkrellmDecal	*mode_decal_button;
+static GkrellmDecal	*option_decal_button;
 
 
 static GkrellmKrell	*krell_in_motion;
@@ -60,21 +61,55 @@ draw_mode_button(gboolean pressed)
 	{
 	gint	frame, x;
 
-	x = gkrellm_chart_width() - decal_mode_button->w + 1;
+	x = gkrellm_chart_width() - mode_decal_button->w + 1;
 	frame = pressed ? D_MISC_BUTTON_IN : D_MISC_BUTTON_OUT;
-	gkrellm_draw_decal_pixmap(NULL, decal_mode_button, frame);
-	gkrellm_draw_decal_on_chart(gkrellmss->chart, decal_mode_button, x, 0);
+	gkrellm_draw_decal_pixmap(NULL, mode_decal_button, frame);
+	gkrellm_draw_decal_on_chart(gkrellmss->chart, mode_decal_button, x, 0);
 	}
 
 static void
-draw_sound_chart(gboolean force_reset, gboolean draw_grid)
+draw_option_button(gboolean pressed)
+	{
+	gint	frame, x;
+
+	x = gkrellm_chart_width() - option_decal_button->w
+				- mode_decal_button->w + 1;
+	frame = pressed ? D_MISC_BUTTON_IN : D_MISC_BUTTON_OUT;
+	gkrellm_draw_decal_pixmap(NULL, option_decal_button, frame);
+	gkrellm_draw_decal_on_chart(gkrellmss->chart, option_decal_button, x, 0);
+	}
+
+void
+gkrellmss_sound_chart_draw(gboolean force_reset, gboolean draw_grid)
 	{
 	if (gkrellmss->mode == SOUND_MODE_OSCOPE)
 		gkrellmss_draw_oscope(force_reset, draw_grid);
 	else if (gkrellmss->mode == SOUND_MODE_SPECTRUM)
 		gkrellmss_draw_spectrum(force_reset, draw_grid);
+
+	if (!gkrellmss->stream_open)
+		{
+		if (gkrellmss->sound_source->type == SOUND_NONE)
+			gkrellm_draw_chart_text(gkrellmss->chart, DEFAULT_STYLE_ID,
+						_("\\c\\fSound source:\\n\\cOff"));
+		else
+			gkrellm_draw_chart_text(gkrellmss->chart, DEFAULT_STYLE_ID,
+						gkrellmss->server_error);
+		}
+
 	if (gkrellmss->mouse_in_chart)
+		{
 		draw_mode_button(gkrellmss->mode_button_pressed);
+		draw_option_button(gkrellmss->option_button_pressed);
+		if (gkrellmss->show_tip)
+			{
+			gkrellm_draw_decal_text(NULL, gkrellmss->label0_decal,
+						gkrellmss->tip_string, -1);
+			gkrellm_draw_decal_on_chart(gkrellmss->chart,
+						gkrellmss->label0_decal, 2,
+						gkrellmss->chart->h - gkrellmss->label0_decal->h - 1);
+			}
+		}
 	gkrellm_draw_chart_to_screen(gkrellmss->chart);
 	gkrellmss->streaming = FALSE;
 	}
@@ -83,6 +118,7 @@ static gint
 cb_chart_release(GtkWidget *widget, GdkEventButton *ev, gpointer data)
 	{
 	gkrellmss->mode_button_pressed = FALSE;
+	gkrellmss->option_button_pressed = FALSE;
 	return TRUE;
 	}
 
@@ -91,14 +127,19 @@ cb_chart_press(GtkWidget *widget, GdkEventButton *ev, gpointer data)
 	{
 	Spectrum	*spectrum = gkrellmss->spectrum;
 
-	if (gkrellm_in_decal(decal_mode_button, ev))
+	if (gkrellm_in_decal(mode_decal_button, ev))
 		{
 		gkrellmss->mode += 1;
 		if (gkrellmss->mode >= N_SOUND_MODES)
 			gkrellmss->mode = 0;
 		gkrellmss->mode_button_pressed = TRUE;
 		gkrellm_config_modified();
-		draw_sound_chart(FORCE_RESET, DRAW_GRID);
+		gkrellmss_sound_chart_draw(FORCE_RESET, DRAW_GRID);
+		}
+	else if (gkrellm_in_decal(option_decal_button, ev))
+		{
+		gkrellmss->option_button_pressed = TRUE;
+		gkrellmss_option_menu(ev);
 		}
 	else if (gkrellmss->stream_open && ev->button == 1)
 		{
@@ -114,13 +155,13 @@ cb_chart_press(GtkWidget *widget, GdkEventButton *ev, gpointer data)
 		{
 		if (!gkrellmss->stream_open)
 			{
-			gkrellmss_sound_open_stream(gkrellmss->host);
+			(*gkrellmss->sound_source->open_stream)();
 			}
 		else
 			{
 			gkrellmss->extra_info = !gkrellmss->extra_info;
 			gkrellm_config_modified();
-			draw_sound_chart(FORCE_RESET, 0);
+			gkrellmss_sound_chart_draw(FORCE_RESET, 0);
 			}
 		}
 	else if (   ev->button == 3
@@ -130,6 +171,55 @@ cb_chart_press(GtkWidget *widget, GdkEventButton *ev, gpointer data)
 	return TRUE;
 	}
 
+static gboolean
+cb_chart_enter(GtkWidget *w, GdkEventButton *ev, gpointer data)
+	{
+	gkrellmss->mouse_in_chart = TRUE;
+	draw_mode_button(FALSE);
+	draw_option_button(FALSE);
+	gkrellm_draw_chart_to_screen(gkrellmss->chart);
+	return TRUE;
+	}
+
+static gint
+cb_chart_leave(GtkWidget *w, GdkEventButton *ev, gpointer data)
+	{
+	gkrellmss->mouse_in_chart = FALSE;
+	gkrellmss->mode_button_pressed = FALSE;
+	gkrellmss->option_button_pressed = FALSE;
+	gkrellmss->show_tip = FALSE;
+	gkrellm_decal_text_clear(gkrellmss->label0_decal);
+	gkrellm_decal_text_clear(gkrellmss->label1_decal);
+	gkrellmss_sound_chart_draw(FORCE_RESET, 0);
+	return TRUE;
+	}
+
+static gint
+cb_chart_motion(GtkWidget *widget, GdkEventButton *ev)
+	{
+	gint	last_show_tip = gkrellmss->show_tip;
+	gchar	*last_string = gkrellmss->tip_string;
+
+	gkrellmss->x_mouse = ev->x;
+
+	gkrellmss->show_tip = TRUE;
+	if (gkrellm_in_decal(mode_decal_button, ev))
+		gkrellmss->tip_string = _("Display mode");
+	else if (gkrellm_in_decal(option_decal_button, ev))
+		gkrellmss->tip_string = _("Options menu");
+	else
+		gkrellmss->show_tip = FALSE;
+
+	if (   last_show_tip != gkrellmss->show_tip
+		|| (   gkrellmss->tip_string && last_string
+			&& strcmp(gkrellmss->tip_string, last_string)
+		   )
+	   )
+		gkrellmss_sound_chart_draw(FORCE_RESET, DRAW_GRID);
+	return TRUE;
+	}
+
+
 static void
 update_sound(void)
 	{
@@ -178,7 +268,7 @@ update_sound(void)
 
 	gkrellm_draw_panel_layers(gkrellmss->chart->panel);
 
-	draw_sound_chart(0, 0);
+	gkrellmss_sound_chart_draw(0, 0);
 	gkrellmss->streaming = FALSE;
 	}
 
@@ -248,32 +338,6 @@ expose_event(GtkWidget *widget, GdkEventExpose *ev)
 	return FALSE;
 	}
 
-
-static gboolean
-cb_chart_enter(GtkWidget *w, GdkEventButton *ev, gpointer data)
-	{
-	gkrellmss->mouse_in_chart = TRUE;
-	draw_mode_button(FALSE);
-	gkrellm_draw_chart_to_screen(gkrellmss->chart);
-	return TRUE;
-	}
-
-static gint
-cb_chart_leave(GtkWidget *w, GdkEventButton *ev, gpointer data)
-	{
-	gkrellmss->mouse_in_chart = FALSE;
-	gkrellmss->mode_button_pressed = FALSE;
-	draw_sound_chart(FORCE_RESET, 0);
-	return TRUE;
-	}
-
-static gint
-cb_chart_motion(GtkWidget *widget, GdkEventButton *ev)
-	{
-	gkrellmss->x_mouse = ev->x;
-	return TRUE;
-	}
-
 static gint
 cb_panel_enter(GtkWidget *widget, GdkEventButton *ev)
 	{
@@ -363,7 +427,7 @@ height_changed(gpointer data)
 		{
 		gkrellmss_load_spectrum_images();
 		}
-	draw_sound_chart(FORCE_RESET, DRAW_GRID);
+	gkrellmss_sound_chart_draw(FORCE_RESET, DRAW_GRID);
 	}
 
 static void
@@ -374,17 +438,22 @@ create_chart(GtkWidget *vbox, gint first_create)
 	/* Create some decals that will be drawn on the chart.  They won't live
 	|  in a panel where they can be automatically destroyed, so I must
     |  destroy them at create events and then recreate them with NULL panel
-	|  and style pointers.
+	|  and style pointers.  Make label0 full chart width so it can hold
+	|  mode/option button "tooltip".
 	*/
 	gkrellm_destroy_decal(gkrellmss->label0_decal);
 	gkrellm_destroy_decal(gkrellmss->label1_decal);
 	gkrellmss->label0_decal = gkrellm_create_decal_text(NULL, "888 msec",
-			gkrellm_chart_alt_textstyle(DEFAULT_STYLE_ID), NULL, 2, 0, 0);
+			gkrellm_chart_alt_textstyle(DEFAULT_STYLE_ID), NULL, 2, 0, -1);
 	gkrellmss->label1_decal = gkrellm_create_decal_text(NULL, "8.8K",
 			gkrellm_chart_alt_textstyle(DEFAULT_STYLE_ID), NULL, 2, 0, 0);
 
-	gkrellm_destroy_decal(decal_mode_button);
-	decal_mode_button = gkrellm_create_decal_pixmap(NULL,
+	gkrellm_destroy_decal(mode_decal_button);
+	gkrellm_destroy_decal(option_decal_button);
+	mode_decal_button = gkrellm_create_decal_pixmap(NULL,
+			gkrellm_decal_misc_pixmap(), gkrellm_decal_misc_mask(),
+			N_MISC_DECALS, NULL, 0, 0);
+	option_decal_button = gkrellm_create_decal_pixmap(NULL,
 			gkrellm_decal_misc_pixmap(), gkrellm_decal_misc_mask(),
 			N_MISC_DECALS, NULL, 0, 0);
 
@@ -417,7 +486,7 @@ create_chart(GtkWidget *vbox, gint first_create)
 		}
 	gkrellmss_oscope_horizontal_scaling();
 	gkrellmss_load_spectrum_images();
-	draw_sound_chart(FORCE_RESET, DRAW_GRID);
+	gkrellmss_sound_chart_draw(FORCE_RESET, DRAW_GRID);
 	}
 
 static void
@@ -436,7 +505,7 @@ cb_sweep_pressed(GkrellmDecalbutton *button)
 		}
 	else if (gkrellmss->mode == SOUND_MODE_SPECTRUM)
 		gkrellmss_change_spectrum_scale(dir);
-	draw_sound_chart(FORCE_RESET, 0);
+	gkrellmss_sound_chart_draw(FORCE_RESET, 0);
 	}
 
 static void
@@ -629,7 +698,7 @@ create_sound(GtkWidget *vbox, gint first_create)
 		{
 		gkrellmss->chart = gkrellm_chart_new0();
 		gkrellmss->chart->panel = gkrellm_panel_new0();
-		gkrellmss_sound_open_stream(gkrellmss->host);
+		(*gkrellmss->sound_source->open_stream)();
 		}
 	gkrellmss->gc = gkrellm_draw_GC(1);
 	create_chart(vbox, first_create);
@@ -648,13 +717,24 @@ save_config(FILE *f)
 	{
 	Oscope		*oscope = gkrellmss->oscope;
 	Spectrum	*spectrum = gkrellmss->spectrum;
+	SoundSource	*source;
+	GList		*list;
 
 	fprintf(f, "%s mode %d\n", CONFIG_KEYWORD, gkrellmss->mode);
 	fprintf(f, "%s sensitivity %f\n", CONFIG_KEYWORD, gkrellmss->vert_sensitivity);
 	fprintf(f, "%s extra_info %d\n", CONFIG_KEYWORD, gkrellmss->extra_info);
 	fprintf(f, "%s usec_per_div %d\n", CONFIG_KEYWORD, oscope->usec_per_div);
 	fprintf(f, "%s spectrum_scale %d\n", CONFIG_KEYWORD,spectrum->scale_index);
+	fprintf(f, "%s sound_source %d\n", CONFIG_KEYWORD,
+					gkrellmss->sound_source_index);
 	gkrellm_save_chartconfig(f, chart_config, CONFIG_KEYWORD, NULL);
+
+	for (list = gkrellmss->sound_source_list; list; list = list->next)
+		{
+		source = (SoundSource *) list->data;
+		if (source->save_config)
+			(*source->save_config)(f, CONFIG_KEYWORD);
+		}
 	}
 
 static void
@@ -662,6 +742,8 @@ load_config(gchar *arg)
 	{
 	Oscope		*oscope = gkrellmss->oscope;
 	Spectrum	*spectrum = gkrellmss->spectrum;
+	SoundSource	*source;
+	GList		*list;
 	gchar		config[32], item[CFG_BUFSIZE];
 	gint		n;
 
@@ -684,8 +766,29 @@ load_config(gchar *arg)
 		sscanf(item, "%d", &oscope->usec_per_div);
 	else if (!strcmp(config, "spectrum_scale"))
 		sscanf(item, "%d", &spectrum->scale_index);
+	else if (!strcmp(config, "sound_source"))
+		{
+		sscanf(item, "%d", &n);
+		list = g_list_nth(gkrellmss->sound_source_list, n);
+		if (!list)
+			{
+			list = gkrellmss->sound_source_list;
+			n = 0;
+			}
+		gkrellmss->sound_source = (SoundSource *) list->data;
+		gkrellmss->sound_source_index = n;
+		}
 	else if (!strcmp(config, GKRELLM_CHARTCONFIG_KEYWORD))
 		gkrellm_load_chartconfig(&chart_config, item, 0);
+	else
+		{
+		for (list = gkrellmss->sound_source_list; list; list = list->next)
+			{
+			source = (SoundSource *) list->data;
+			if (source->load_config && !strcmp(config, source->name))
+				(*source->load_config)(item);
+			}
+		}
 	}
 
 /* --------------------------------------------------------------------- */
@@ -765,7 +868,7 @@ create_tab(GtkWidget *tab_vbox)
 	gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
 
 	buf = g_strdup_printf(_("GKrellMSS %d.%d %s\nGKrellM Sound Scope\n\n"
-				"Copyright (c) 2002 by Bill Wilson\n"
+				"Copyright (c) 2002-2004 by Bill Wilson\n"
 				"bill@gkrellm.net\n"
 				"http://gkrellm.net\n\n"
 				"Released under the GNU Public License"),
@@ -820,6 +923,15 @@ gkrellm_init_plugin(void)
 	gkrellmss->extra_info = TRUE;
 	gkrellmss->vert_sensitivity = 0.5;
 
+	gkrellmss_add_sound_sources();
+	if (!gkrellmss->sound_source_list)
+		return NULL;
+	gkrellmss_option_menu_build();
+
+	gkrellmss->sound_source =
+			(SoundSource *) gkrellmss->sound_source_list->data;
+	gkrellmss->sound_source_index = 0;
+
 	gkrellmss->oscope = gkrellmss_init_oscope();
 
 	gkrellmss->spectrum = gkrellmss_init_spectrum();
@@ -827,11 +939,10 @@ gkrellm_init_plugin(void)
 	style_id = gkrellm_add_meter_style(mon_sound, STYLE_NAME);
 
 	gkrellm_locale_dup_string(&dummy_utf8,
-			_("\\cNo sound server?\\n\\f\\cClick here to\\n\\f\\ctry to open"),
+			_("\\cSound error?\\n\\f\\cClick here to\\n\\f\\ctry to open"),
 			&gkrellmss->server_error);
 	g_free(dummy_utf8);
 
-
 	return &monitor_sound;
 	}
 
