@@ -1,5 +1,5 @@
 /* GKrellM Sound Scope
-|  Copyright (C) 2002 Bill Wilson
+|  Copyright (C) 2002-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -22,8 +22,13 @@
 
 #include <gkrellm2/gkrellm.h>
 #include <math.h>
-#include <rfftw.h>
+#include "configure.h"
 
+#if defined(HAVE_FFTW3)
+#include <fftw3.h>
+#elif defined(HAVE_FFTW2)
+#include <rfftw.h>
+#endif
 
 #if  !defined(GKRELLM_VERSION_MAJOR) || (GKRELLM_VERSION_MAJOR<2)
 #error This GKrellMSS plugin requires GKrellM version >= 2.0.0
@@ -31,7 +36,7 @@
 
 
 #define	GKRELLMSS_VERSION_MAJOR	2
-#define	GKRELLMSS_VERSION_MINOR	3
+#define	GKRELLMSS_VERSION_MINOR	6
 #define	GKRELLMSS_EXTRAVERSION ""
 
 #define CONFIG_NAME     "Sound Scope"
@@ -87,9 +92,15 @@ typedef struct
 typedef struct
 	{
 	gint		fftw_samples;
+#if defined(HAVE_FFTW2)
 	fftw_real	*fftw_data_in,
 				*fftw_data_out,
 				*fftw_power_spectrum;
+#elif defined(HAVE_FFTW3)
+	double		*fftw_data_in,
+				*fftw_data_out,
+				*fftw_power_spectrum;
+#endif
 	GdkPixmap	*bar,
 				*bar_light;
 	gint		scale_index;
@@ -119,12 +130,34 @@ typedef struct
 Oscope;
 
 
+#define	SOUND_NONE		1
+#define	SOUND_CARD		2
+#define SOUND_SERVER	3
+
+typedef struct
+	{
+	gchar	*name;
+	gint	type;
+	gchar	*preselect_item;
+
+	void	(*open_stream)();	
+	void	(*close_stream)();	
+	void	(*option_menu)();	
+	void	(*load_config)();
+	void	(*save_config)();
+	}
+	SoundSource;
+
+
 typedef struct
 	{
 	Oscope		*oscope;
 	Spectrum	*spectrum;
 	gint		mode;
 	GdkGC		*gc;
+	GList		*sound_source_list;
+	SoundSource	*sound_source;
+	gint		sound_source_index;
 
 	GkrellmChart *chart;
 	GkrellmDecal *label0_decal,
@@ -149,7 +182,6 @@ typedef struct
 				krell_sensitivity_y_dn;
 	gfloat		vert_sensitivity;
 
-	gchar		*host;
 	gint		fd;
 	gpointer	handle;
 	gint		input_id;
@@ -157,7 +189,11 @@ typedef struct
 				streaming,
 				extra_info,
 				mouse_in_chart,
-				mode_button_pressed;
+				mode_button_pressed,
+				option_button_pressed,
+				show_tip;
+	gchar		*tip_string;
+
 	gint		x_mouse;
 	gchar		*server_error;
 
@@ -169,7 +205,6 @@ typedef struct
 	SoundMonitor;
 
 
-
 #define LOW_SWEEP   100
 #define HIGH_SWEEP  50000
 
@@ -180,8 +215,8 @@ typedef struct
 SoundMonitor	*gkrellmss;
 
 
-void		gkrellmss_sound_open_stream(gchar *);
-void		gkrellmss_sound_close_stream(void);
+void		gkrellmss_add_sound_sources(void);
+void		gkrellmss_sound_chart_draw(gboolean reset, gboolean draw_grid);
 
 /* oscope */
 void		gkrellmss_draw_oscope(gboolean, gboolean);
@@ -197,5 +232,5 @@ void		gkrellmss_spectrum_alloc_data(void);
 Spectrum	*gkrellmss_init_spectrum(void);
 
 
-/* grab from gkrellm-private.h, will be in gkrellm-public-proto.h in 2.0.1 */
-gboolean gkrellm_locale_dup_string(gchar **dst, gchar *src, gchar **locale);
+void		gkrellmss_option_menu(GdkEventButton *ev);
+void		gkrellmss_option_menu_build(void);
