@@ -1,5 +1,5 @@
 /* GKrellMSS - GKrellM Sound Scope
-|  Copyright (C) 2002 Bill Wilson
+|  Copyright (C) 2002-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |
@@ -233,10 +233,9 @@ draw_oscope_labels(void)
 	{
 	GkrellmChart	*chart	= gkrellmss->chart;
 
-	if (!gkrellmss->stream_open)
-		gkrellm_draw_chart_text(chart, DEFAULT_STYLE_ID,
-					gkrellmss->server_error);
-	else if (gkrellmss->extra_info)
+	if (gkrellmss->show_tip)
+		return;
+	if (gkrellmss->stream_open && gkrellmss->extra_info)
 		{
 		draw_oscope_label_decals();
 		gkrellm_draw_decal_on_chart(chart, gkrellmss->label0_decal, 2,
