@@ -1,5 +1,5 @@
 /* GKrellMSS - GKrellM Sound Scope
-|  Copyright (C) 2002 Bill Wilson
+|  Copyright (C) 2002-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |
@@ -29,10 +29,22 @@ static fftw_plan	plan_1024, plan_2048, plan_4096, plan_8192;
 static gint		debug_trigger;
 
 
+  /* Process halfcomplex double data output from executing the r2r fft:
+  |  	r[0],r[1],r2,...,r[n/2},i[(n+1)/2 - 1],...,i[2],i[1]
+  |
+  |  r[k] and i[k] are real and imaginary part of kth output.  So for an
+  |  output array size N, the kth real part is output[k] and the kth
+  |  imaginary part is output[N-k], except for k == 0 (and k == N/2 if
+  |  N is even).
+  */
 static void
 process_fftw_data(void)
 	{
+#if defined(HAVE_FFTW3)
+	double		*in, *out, *ps;
+#else
 	fftw_real	*in, *out, *ps;
+#endif
 	gint		k, N;
 
 	in = spectrum->fftw_data_in;
@@ -40,7 +52,7 @@ process_fftw_data(void)
 	ps = spectrum->fftw_power_spectrum;
 	N = spectrum->fftw_samples;
 
-	ps[0] = out[0] * out[0];		/* DC component */
+	ps[0] = out[0] * out[0];		/* r[0] * r[0] DC component */
 	for (k = 1; k < (N+1)/2; ++k)	/* (k < N/2 rounded up) */
 		{
 		ps[k] = out[k] * out[k] + out[N-k] * out[N-k];	/* Nyquist freq. */
@@ -147,19 +159,20 @@ draw_spectrum_decal_label(GkrellmDecal *d, gfloat f, gint x, gint y)
 static void
 draw_spectrum_labels(void)
 	{
-	GkrellmChart	*chart	= gkrellmss->chart;
 	SpectrumScale	*scale = spectrum->scale;
 	gfloat			freq0 = 0, freq1 = 0;
 
-	if (gkrellmss->fd < 0)
-		gkrellm_draw_chart_text(chart, DEFAULT_STYLE_ID,
-					gkrellmss->server_error);
-	else if (spectrum->freq_highlighted > 0)
-		freq0 = spectrum->freq_highlighted;
-	else if (gkrellmss->extra_info)
+	if (gkrellmss->show_tip)
+		return;
+	if (gkrellmss->stream_open)
 		{
-		freq0 = scale->freq_array[scale->start_bar];
-		freq1 = scale->freq_array[scale->array_size - 2];
+		if (spectrum->freq_highlighted > 0)
+			freq0 = spectrum->freq_highlighted;
+		else if (gkrellmss->extra_info)
+			{
+			freq0 = scale->freq_array[scale->start_bar];
+			freq1 = scale->freq_array[scale->array_size - 2];
+			}
 		}
 	if (freq0 > 0)
 		draw_spectrum_decal_label(gkrellmss->label0_decal, freq0, 1, 1);
@@ -187,9 +200,12 @@ set_bar_frequency(SpectrumScale *scale, gint *k, double *flog, double dflog)
 	*/
 	qhits = fr / scale->freq_quantum - fl / scale->freq_quantum;
 
-	if (DEBUG() && ftest < 100)
-		printf("l=%d test=%d fl=%.1f fr=%.1f hits=%.1f\n", l, ftest, fl,
-				fr, qhits);
+	if (DEBUG() && ftest < 100)  /* look at small ftest band or lotsa output */
+//	if (DEBUG() && ftest > 15000)
+		printf(
+"bar[%d-%s] l=%d ftest=%d fl=%.1f fr=%.1f hits=%.1f freq_quantum=%.1f\n",
+			*k, (qhits > 1.0) ? "yes" : " no",
+			l, ftest, fl, fr, qhits, scale->freq_quantum);
 
 	if (   *k < scale->array_size - 1
 		&& qhits > 1.0
@@ -205,7 +221,8 @@ set_bar_frequency(SpectrumScale *scale, gint *k, double *flog, double dflog)
 	return FALSE;
 	}
 
-  /* I construct the frequency table around power of 10 frequencies.
+  /* Construct the frequency table around power of 10 frequencies. So 100, 1k,
+  |  and 10K bars will be on the chart.
   |  Fixme so f_min_10 is calculated and handle case for no f_min_10.
   */
 static void
@@ -291,7 +308,11 @@ gkrellmss_draw_spectrum(gboolean force_reset, gboolean draw_grid)
 	SpectrumScale	*scale;
 	gint			*freq_table;
 	gint			n, k, N, i;
+#if defined(HAVE_FFTW3)
+	double			*ps;
+#else
 	fftw_real		*ps;
+#endif
 	double			m, f, flimit;
 	gint			x, y, dy;
 	gboolean		highlight;
@@ -315,7 +336,12 @@ gkrellmss_draw_spectrum(gboolean force_reset, gboolean draw_grid)
 	if (spectrum->fftw_samples != scale->n_samples)
 		return;
 
+#if defined(HAVE_FFTW3)
+	fftw_execute(*(scale->plan));
+#elif defined(HAVE_FFTW2)
 	rfftw_one(*(scale->plan), spectrum->fftw_data_in, spectrum->fftw_data_out);
+#endif
+
 	process_fftw_data();
 
 	gkrellm_clear_chart_pixmap(chart);
@@ -377,7 +403,7 @@ gkrellmss_draw_spectrum(gboolean force_reset, gboolean draw_grid)
 				}
 			if (n > 0)
 				{
-				m = sqrt(m) / 20;
+				m = sqrt(m) / (scale->n_samples / 200);
 				dy = m * chart->h / spectrum->vert_max;
 				if (dy > chart->h)
 					dy = chart->h;
@@ -406,7 +432,7 @@ gkrellmss_draw_spectrum(gboolean force_reset, gboolean draw_grid)
 	}
 
 
-static SpectrumScale scale_table[4];
+static SpectrumScale scale_table[5];
 
 #define N_SPECTRUM_SCALES (sizeof(scale_table) / sizeof(SpectrumScale))
 
@@ -428,13 +454,14 @@ gkrellmss_load_spectrum_images(void)
 		*/
 		/* Fast response, so visually nicer:
 		*/
-		load_freq_array(&scale_table[0], 20, 25000, 100, 0, 2, 2048);
+		load_freq_array(&scale_table[0], 20, 25000, 100, 0, 2, 1024);
+		load_freq_array(&scale_table[1], 20, 25000, 100, 0, 2, 2048);
 
 		/* Better and more interesting low frequency data, but slower:
 		*/
-		load_freq_array(&scale_table[1], 22, 20000, 100, 0, 2, 4096);
-		load_freq_array(&scale_table[2], 18, 20000, 100, 2, 1, 8192);
-		load_freq_array(&scale_table[3], 10, 3000, 100, 0, 2, 8192);
+		load_freq_array(&scale_table[2], 22, 20000, 100, 0, 2, 4096);
+		load_freq_array(&scale_table[3], 18, 20000, 100, 2, 1, 8192);
+		load_freq_array(&scale_table[4], 10, 3000, 100, 0, 2, 8192);
 		}
 	last_w = w;
 	gkrellm_load_piximage("spectrum_bar", spectrum_bar_xpm, &im, STYLE_NAME);
@@ -471,6 +498,35 @@ gkrellmss_spectrum_alloc_data(void)
 		return;
 
 	spectrum->scale = &scale_table[0];
+
+#if defined(HAVE_FFTW3)
+	spectrum->fftw_data_in =
+				(double *) fftw_malloc(N_FFT_SAMPLES * sizeof(double));
+	spectrum->fftw_data_out =
+				(double *) fftw_malloc(N_FFT_SAMPLES * sizeof(double));
+	spectrum->fftw_power_spectrum =
+				(double *) fftw_malloc(N_FFT_SAMPLES * sizeof(double));
+
+	/* Use fftw3 real to real transform of kind FFTW_R2HC which takes a real
+	|  double array as input and outputs a double array of the same size.
+	|  This computes a real input DFT with output in "halfcomplex" format:
+	|
+	|      r[0],r[1],r2,...,r[n/2},i[(n+1)/2 - 1],...,i[2],i[1]
+	|
+	|  So r[0] is the DC component.  See process_fftw_data().
+	*/
+	plan_1024 = fftw_plan_r2r_1d(1024, spectrum->fftw_data_in,
+					spectrum->fftw_data_out, FFTW_R2HC, 0);
+	plan_2048 = fftw_plan_r2r_1d(2048, spectrum->fftw_data_in,
+					spectrum->fftw_data_out, FFTW_R2HC, 0);
+	plan_4096 = fftw_plan_r2r_1d(4096, spectrum->fftw_data_in,
+					spectrum->fftw_data_out, FFTW_R2HC, 0);
+	plan_8192 = fftw_plan_r2r_1d(8192, spectrum->fftw_data_in,
+					spectrum->fftw_data_out, FFTW_R2HC, 0);
+
+	/* Ok, no more plans because N_FFT_SAMPLES is 8192 */
+
+#elif defined(HAVE_FFTW2)
 	spectrum->fftw_data_in = g_new0(fftw_real, N_FFT_SAMPLES);
 	spectrum->fftw_data_out = g_new0(fftw_real, N_FFT_SAMPLES);
 	spectrum->fftw_power_spectrum = g_new0(fftw_real, N_FFT_SAMPLES);
@@ -479,6 +535,8 @@ gkrellmss_spectrum_alloc_data(void)
 	plan_2048 = rfftw_create_plan(2048, FFTW_REAL_TO_COMPLEX, FFTW_ESTIMATE);
 	plan_4096 = rfftw_create_plan(4096, FFTW_REAL_TO_COMPLEX, FFTW_ESTIMATE);
 	plan_8192 = rfftw_create_plan(8192, FFTW_REAL_TO_COMPLEX, FFTW_ESTIMATE);
+#endif
+
 	}
 
 Spectrum *
